#!/usr/bin/python
# 3D MRI - Converts a series of slices into a 3D grid of voxels.
# Render a series of slices on other axies.
# Copyright (C) November 2007 Neil Fraser
# http://neil.fraser.name/

# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# http://www.gnu.org/

import gd
import sys
import os
import file_lib

if len(sys.argv) != 2:
  print "Usage: %s <source directory>" % sys.argv[0]
  sys.exit()

dir = sys.argv[1]
if not os.path.exists(dir):
  print "Directory not found: %s" % dir
  sys.exit()

# Identify all the images.
print "Identifying images in %s" % dir
x = None
y = None
files = os.listdir(dir)
files.sort()
picpaths = []
for file in files:
  picpath = os.path.join(dir, file)
  if (os.path.isfile(picpath)):
    try:
      image = gd.image(picpath)
    except IOError:
      image = None
    if image:
      if x == None and y == None:
        # First image
        (x, y) = image.size()
      elif (x, y) != image.size():
        print "\nImage not the same dimensions as previous images: %s" % picpath
        sys.exit()
      sys.stdout.write(".")
      sys.stdout.flush()
      picpaths.append(picpath)
    else:
      print "\nSkipping non-image: " + picpath

z = len(picpaths)
if z == 0:
  print "Zero images found in %s" % dir
  sys.exit()
print "Done.\n"

# Build an empty 3D array of the correct size.
# Python is not the greatest language for this.
print "Initializing the image space (%d x %d x %d)" % (x, y, z)
a = []
for k in xrange(z):
  sys.stdout.write(".")
  sys.stdout.flush()
  a.append([])
  for j in xrange(y):
    a[k].append(range(x))
print "Done.\n"

print "Reading the %d image slices" % z
for k in xrange(z):
  sys.stdout.write(".")
  sys.stdout.flush()
  image = gd.image(picpaths[k])
  for j in xrange(y):
    for i in xrange(x):
      colourIndex = image.getPixel((i, j))
      red = image.red(colourIndex)
      green = image.green(colourIndex)
      blue = image.blue(colourIndex)
      grey = (red + green + blue) / 3.0
      a[k][j][i] = grey / 255.0
print "Done.\n"

# Save data to disk.
file_lib.save_data(dir, "data.txt", a)

# Verify the data (optional)
#b = file_lib.load_data(dir, "data.txt")
#print "Verifying data (%d x %d x %d)" % file_lib.get_xyz(b)
#for k in xrange(z):
#  sys.stdout.write(".")
#  sys.stdout.flush()
#  for j in xrange(y):
#    for i in xrange(x):
#      # Floats often don't equal themselves.  Only check first 4 digits.
#      if int(a[k][j][i] * 1000) != int(b[k][j][i] * 1000):
#        print "Failed at (%d x %d x %d): %f != %f\n" % (i, j, k, a[k][j][i], b[k][j][i])
#print "Done.\n"
