#!/usr/bin/python
# 3D MRI - Converts a series of slices into a 3D grid of voxels.
# Render a series of slices on other axies.
# Copyright (C) November 2007 Neil Fraser
# http://neil.fraser.name/

# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# http://www.gnu.org/

import sys
import os
import file_lib

if len(sys.argv) != 2:
  print "Usage: %s <source datafile>" % sys.argv[0]
  sys.exit()

datapath = sys.argv[1]
a = file_lib.load_data("", datapath)
(x, y, z) = file_lib.get_xyz(a)

print "Shifting frames to black"
for k in xrange(z):
  smallest = 1.0
  for j in xrange(y):
    for i in xrange(x):
      smallest = min(smallest, a[k][j][i])
  print "%d: %f" % (k, smallest)
  for j in xrange(y):
    for i in xrange(x):
      a[k][j][i] -= smallest
print "Done.\n"

largest = 0.0
print "Searching for largest value"
for k in xrange(z):
  sys.stdout.write(".")
  sys.stdout.flush()
  for j in xrange(y):
    for i in xrange(x):
      largest = max(largest, a[k][j][i])
print "\nLargest value: %f" % largest

print "Stretching block to white"
for k in xrange(z):
  sys.stdout.write(".")
  sys.stdout.flush()
  for j in xrange(y):
    for i in xrange(x):
      a[k][j][i] /= largest
print "Done.\n"

file_lib.save_data("", datapath, a)
