#!/usr/bin/python
# 3D MRI - Converts a series of slices into a 3D grid of voxels.
# Render a series of slices on other axies.
# Copyright (C) November 2007 Neil Fraser
# http://neil.fraser.name/

# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# http://www.gnu.org/

import sys
import os
import file_lib
import gd

if len(sys.argv) != 3:
  print "Usage: %s <source datafile> <scale>" % sys.argv[0]
  sys.exit()

scale = float(sys.argv[2])

datapath = sys.argv[1]
a = file_lib.load_data("", datapath)
(x, y, z) = file_lib.get_xyz(a)


# Obtain a colour in the colour palette of this image.
def getColourIndex(image, colour):
  colour = int(colour * 255 + 0.5)
  colour = (colour, colour, colour)
  colourIndex = image.colorExact(colour)
  if colourIndex == -1:
    colourIndex = image.colorAllocate(colour)
    if colourIndex == -1:
      colourIndex = image.colorClosest(colour)
      print "\nWarning, out of colour space."
  assert colourIndex != -1, "Failed to allocate colour"
  return colourIndex


# Resize the image.
def resizeImage(image, smallx, smally):
  global scale
  if scale == 1.0:
    return image
  bigx = int(smallx * scale + 0.5)
  bigy = int(smally * scale + 0.5)
  bigimage = gd.image((bigx, bigy))
  image.copyResampledTo(bigimage, (0, 0), (0, 0), (bigx, bigy))
  return bigimage


print "Writing the RL image slices"
for k in xrange(0, z):
  sys.stdout.write(".")
  sys.stdout.flush()
  image = gd.image((x, y))
  for j in xrange(0, y):
    for i in xrange(0, x):
      colourIndex = getColourIndex(image, a[k][j][i])
      image.setPixel((i, j), colourIndex)
  image = resizeImage(image, x, y)
  image.writePng("RL/RL_%s.png" % ("000" + str(k))[-3:])
print "Done.\n"

print "Writing the TB image slices"
for j in xrange(0, y):
  sys.stdout.write(".")
  sys.stdout.flush()
  image = gd.image((x, z))
  for i in xrange(0, x):
    for k in xrange(0, z):
      colourIndex = getColourIndex(image, a[k][j][i])
      image.setPixel((i, k), colourIndex)
  image = resizeImage(image, x, z)
  image.writePng("TB/TB_%s.png" % ("000" + str(j))[-3:])
print "Done.\n"

print "Writing the FB image slices"
for i in xrange(0, x):
  sys.stdout.write(".")
  sys.stdout.flush()
  image = gd.image((z, y))
  for k in xrange(0, z):
    for j in xrange(0, y):
      colourIndex = getColourIndex(image, a[k][j][i])
      image.setPixel((k, j), colourIndex)
  image = resizeImage(image, z, y)
  image.writePng("FB/FB_%s.png" % ("000" + str(i))[-3:])
print "Done.\n"
