#!/usr/bin/python
# 3D MRI - Converts a series of slices into a 3D grid of voxels.
# Render a series of slices on other axies.
# Copyright (C) November 2007 Neil Fraser
# http://neil.fraser.name/

# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# http://www.gnu.org/

import sys
import os
import file_lib

if len(sys.argv) != 2:
  print "Usage: %s <source datafile>" % sys.argv[0]
  sys.exit()

datapath = sys.argv[1]
a = file_lib.load_data("", datapath)
(x, y, z) = file_lib.get_xyz(a)

prefix = """
#include "colors.inc"
// How far back is the camera; drags the lights along with it.
#declare camera_back = 250;
// Front-Back, Top-Bottom and Right-Left distances
#declare fb = %d;
#declare tb = %d;
#declare rl = %d;
global_settings { ambient_light rgb<400, 300, 300>}
union {
""" % (x, y, z)

suffix = """
  translate <-rl/2,-tb/2,-fb/2>
  rotate <0,clock,0>
}

// Look from slightly above.
camera {
  location <0, 0, -camera_back>
  look_at  <0, 0, 0>
}
// Offset light
light_source {
  <-camera_back, camera_back, -camera_back>
  color White
}
"""

povfile = "3d.pov"
print "Writing %s" % povfile

file = open(povfile, "w")
file.write(prefix)

fadeFactor = 100.0 # must be float
voxelUsed = 0
voxelLoss = 0
for k in xrange(0, z):
  sys.stdout.write(".")
  sys.stdout.flush()
  for j in xrange(0, y):
    for i in xrange(0, x):
      if a[k][j][i] > 0.1:
        opacity = 1.0 - (a[k][j][i] / fadeFactor)
        file.write("  box {<%d,%d,%d>, <%d,%d,%d> texture {pigment {color rgbt <1,1,1,%f>}}}\n" %
            (k, y-j-1, i, k+1, y-j, i+1, opacity))
        voxelUsed += 1
      else:
        voxelLoss += 1
file.write(suffix)
file.close()
print "Done.\n"

print "%d voxels used (%f%%)" % (voxelUsed, voxelUsed * 100 / (voxelUsed + voxelLoss))
